@@ -6,9 +6,7 @@ use base 'Class::Component::Attribute';
 sub register {
     my ( $class, $plugin, $c, $method, $carrier, $code ) = @_;
 
-    if ($c->carrier_longname eq $carrier) {
-        $c->register_method( $method => $plugin );
-    }
+    $c->agent_class($carrier)->register_method( $method => $plugin );
 }
 
 1;
