@@ -3,10 +3,10 @@ use strict;
 use warnings;
 use base qw/HTTP::MobileAttribute::Plugin/;
 
-sub initialize : Hook('initialize') {
+sub initialize : Hook('initialize_AirHPhone') {
     my ($self, $c) = @_;
-    return unless $c->carrier_longname eq 'AirHPhone';
-    $self->mk_register_accessors( $c => qw(name vendor model model_version browser_version cache_size));
+
+    $self->mk_register_accessors( AirHPhone => qw(name vendor model model_version browser_version cache_size));
     $self->parse($c->user_agent);
 }
 
